/*
 * This file is part of din.
 *
 * din is copyright (c) 2006 - 2012 S Jagannathan <jag@dinisnoise.org>
 * For more information, please visit http://dinisnoise.org
 *
 * din is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * din is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with din.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
#include "beat2value.h"
#include "console.h"
#include <fstream>
#include <iostream>
using namespace std;

extern console cons;

beat2value::beat2value (const string& c) : crvname (c), crv (c)  {

  now = 0;
  delta = 0;
  bpm = 45;

  set_bpm (bpm);
  sol(&crv);
  curve_edited ();

  style = "loop";
  xmin = &_loopmin;
  xmax = &_loopmax;

}

beat2value::~beat2value () {
  crv.save (crvname);
}

void beat2value::operator() (float* soln, int n) {
  sol (now, delta, n, soln, *xmin, *xmax);
}

void beat2value::operator() (float* out, float* mul, int n) {
  for (int i = 0; i < n; ++i) out[i] *= mul[i];
}

void beat2value::modulate (float* out, float* mod, int n, float depth) {
  sol (now, delta, n, mod, *xmin, *xmax);
  for (int i = 0; i < n; ++i) out[i] = (depth * mod [i]);
}

void beat2value::curve_edited () {
  sol.update ();
}

void beat2value::set_bpm (float n) {
  if (n < 0) return;
  bpm = n;
  float bps = bpm / 60.0;
  int sgn = 1; if (delta < 0) sgn = -1;
  extern int SAMPLE_RATE;
  delta = sgn * bps * 1. / SAMPLE_RATE;
}

float beat2value::get_bpm () { return bpm; }
