/*
 * This file is part of din.
 *
 * din is copyright (c) 2006 - 2012 S Jagannathan <jag@dinisnoise.org>
 * For more information, please visit http://dinisnoise.org
 *
 * din is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * din is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with din.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
#include "ui_list.h"
#include "din.h"
#include "morse_code.h"
#include "console.h"
#include "curve_editor.h"
#include "tcl_interp.h"
#include "command.h"
#include "chrono.h"
#include "font_editor.h"
#include "key_consts.h"
#include "keyboard_keyboard.h"
#include "fft.h"

using namespace std;

extern console cons;
extern cmdlist cmdlst;
extern din din0;
extern morse_code mc;
extern curve_editor delayed;
extern string dotdin;
extern tcl_interp* p_interp;
extern font fnt;
extern curve_editor waved, comed, octed, veled, dmod_ed;

extern keyboard_keyboard keybd2;
extern fft fft0;

ui_list::ui_list () :  fed (fnt, "font.ed") {

  current = 0;
  last = 0;

  crved = 0;

  uis.push_back (&din0);
  uis.push_back (&din0.waved);
  uis.push_back (&keybd2.waved);
  uis.push_back (&din0.moded);
  uis.push_back (&din0.gated);
  uis.push_back (&delayed);
  uis.push_back (&octed);
  uis.push_back (&din0.droneed);
  uis.push_back (&comed);
  uis.push_back (&mc.ed);
  uis.push_back (&dmod_ed);
  uis.push_back (&fed);
  uis.push_back (&keybd2.attacked);
  uis.push_back (&keybd2.decayed);
  uis.push_back (&keybd2.veled);

  esct = -1;

}

void ui_list::set_current (ui* u, int ised) {

  if (current) {
    current->leave ();
    last = current;
  }

  current = u;
  current->enter ();

  if (ised) {
    crved = dynamic_cast<curve_editor*>(u);
    if (crved->sine_enabled) fft0.go (crved->curveinfo[0].curve);
  } else {
    crved = 0;
    uis[0] = current;
  }

  cons << console::yellow << "@ " << current->name << eol;

}

bool ui_list::handle_input () {

  current->calc_win_mouse ();

  extern din din0; din0.bg ();

  for (int i = 1, j = uis.size(); i < j; ++i) uis[i]->bg ();

  if (cons.command_mode == 0) {

    current->handle_input ();

    if (keypressed (k_enter)) { // calc tap bpm
      extern nano_chrono ui_clk;
      extern double TAP_BPM;
      static double last_tapt = ui_clk.secs;
      double dt = ui_clk.secs - last_tapt;
      if (dt) TAP_BPM = 60.0 / dt;
      extern tcl_interp interpreter;
      if (keydown (k_lctrl)) cons ("set resetbeat 1");
      Tcl_UpdateLinkedVar (interpreter.interp, "tapbpm");
      last_tapt = ui_clk.secs;
    }

    extern string INSTRUMENT;

    if (keypressed (k_1)) {
      if (crved == 0) {
        if (current == &din0) {
          set_current (&keybd2, 0);
          cons ("setup-editors keyboard_keyboard");
          INSTRUMENT = "keyboard_keyboard";
        } else {
          set_current (&din0, 0);
          cons ("setup-editors microtonal_keyboard");
          INSTRUMENT = "microtonal_keyboard";
        }
      } else set_current (uis[0], 0);
    }

    for (int i = 1; i < MAX_EDITORS; ++i) {
      if (keypressed (key[i])) {
        if (ed[i]) set_current (ed[i]);
        break;
      }
    }

  }

  cons.handle_input ();

  extern nano_chrono ui_clk;
  if (keypressed (k_esc)) {
    if (cons.command_mode || (current != uis[0])) {
      set_current (uis[0], 0);
      cons.command_mode = 0;
      esct = -1;
    } else {
      esc:
      if (esct == -1) {
        cons << console::red << "Press ESC again to quit" << eol;
        esct = ui_clk.secs;
      } else {
        static const float ESC_TIMEOUT = 1; // in seconds
        float dt = ui_clk.secs - esct;
        if (dt > ESC_TIMEOUT) {
          esct = -1;
          goto esc;
        } else {
          extern int quit_now; quit_now = 1;
        }
      }
    }
  }

  return true;
}

bool ui_list::set_editor (const string& name, int screen) {

  if (screen > 0) {

    ui* ei = 0;
    for (int i = 0, j = uis.size(); i < j; ++i) {
      if (uis[i]->name == name) {
        ei = uis[i];
        break;
      }
    }
    if (ei) {
      ed [screen - 1] = ei;
      return true;
    } else {
      cmdlst.result = "bad editor name";
      return false;
    }
  } else {
    cmdlst.result = "bad screen number (valid: 1 to 8)";
    return true;
  }
}

void ui_list::draw () {
  glClear (GL_COLOR_BUFFER_BIT);
  current->draw ();
  cons.draw ();
}
